
import 'package:chobi_admob_flutter/chobi_admob.dart';
import 'package:flutter/widgets.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

class ChobiAdMobAdaptiveBanner extends StatefulHookConsumerWidget {

  final String? androidBannerId;
  final String? iosBannerId;
  final String lastClickedPrefsKey;
  final List<String>? testDeviceIds;
  final bool isDebug;
  final int adIntervalTimeMs;

  final Function(ConsentStatus status)? additionalConsentCheckFinished;
  final Function(FormError error)? additionalConsentError;

  const ChobiAdMobAdaptiveBanner({
    Key? key,
    required this.lastClickedPrefsKey,
    this.androidBannerId,
    this.iosBannerId,
    this.testDeviceIds,
    this.isDebug = true,
    this.adIntervalTimeMs = 5 * 60 * 1000,
    this.additionalConsentCheckFinished,
    this.additionalConsentError,
  }) : super(key: key);

  @override
  ConsumerState<ChobiAdMobAdaptiveBanner> createState() => _ChobiAdMobAdaptiveBannerState();
}

class _ChobiAdMobAdaptiveBannerState extends ConsumerState<ChobiAdMobAdaptiveBanner> {

  @protected
  final isAdAvailableProvider = StateProvider((ref) => false);

  @protected
  final isAdLoadedProvider = StateProvider((ref) => false);

  final _bannerAd = StateProvider<BannerAd?>((ref) => null);

  late ChobiAdMob? _chobiAdMob;

  @protected
  Future<void> loadBanner() async {

    if (!ref.read(isAdAvailableProvider)) {
      debugPrint('AD not available');
      return;
    }

    final bannerId = _chobiAdMob?.getAdId();

    if (bannerId == null) {
      debugPrint('banner id is null');
      return;
    }

    final cWidth = MediaQuery.of(context).size.width.truncate();
    final adSize = await AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(cWidth);

    if (adSize == null) {
      debugPrint('adSize is null');
      return;
    }

    final ad = BannerAd(
      size: adSize,
      adUnitId: bannerId,
      listener: generateBannerListener(),
      request: const AdRequest(),
    );
    ref.read(_bannerAd.notifier).state = ad;
    await ad.load();
  }


  @protected
  Future<void> disposeBanner() async {
    final ad = ref.read(_bannerAd);
    if (ad != null) {
      await ad.dispose();
    }
    ref.read(_bannerAd.notifier).state = null;
  }

  void _onConsentCheckFinished(ConsentStatus status) {
    if (status != ConsentStatus.required) {
      ref.read(isAdAvailableProvider.notifier).state = true;
      loadBanner();
    }
  }

  void _onConsentError(FormError error) {
    ref.read(isAdAvailableProvider.notifier).state = false;
    debugPrint('$error');
  }

  @override
  void initState() {
    super.initState();

    ChobiAdMob.loadLastClickedTimeFromPrefs(prefKey: widget.lastClickedPrefsKey).then((value) {
      _chobiAdMob = ChobiAdMob(
        adLastClickedTime: value,
        lastClickedPrefsKey: widget.lastClickedPrefsKey,
        isDebug: widget.isDebug,
        testDeviceIds: widget.testDeviceIds,
        androidId: widget.androidBannerId,
        iosId: widget.iosBannerId,
        androidTestId: ChobiAdMob.androidTestAdaptiveBannerId,
        iosTestId: ChobiAdMob.iosTestAdaptiveBannerId,
        adIntervalTimeMs: widget.adIntervalTimeMs,
      )..onConsentCheckFinished =  (status) {
        _onConsentCheckFinished(status);
        widget.additionalConsentCheckFinished?.call(status);
      }..onConsentCheckError = (err) {
        _onConsentError(err);
        widget.additionalConsentError?.call(err);
      }..checkPermissionAndStartAdProcess();
    });
  }

  @override
  void dispose() {
    Future.sync(() async {
      await _chobiAdMob?.stopAdProcess();
      await disposeBanner();
    });

    super.dispose();
  }


  final _blankBox = const SizedBox.shrink();

  @protected
  BannerAdListener generateBannerListener() {
    return BannerAdListener(
      onAdLoaded: (ad) {
        ref.read(isAdLoadedProvider.notifier).state = true;
      },
      onAdFailedToLoad: (ad, err) {
        debugPrint('admob banner laod error: ${err.message}');
        ref.read(isAdLoadedProvider.notifier).state = false;
        disposeBanner();
      },
      onAdClicked: (ad) async {
        await _chobiAdMob?.updateLastClickedTime();
        await disposeBanner();
        await _chobiAdMob?.checkPermissionAndStartAdProcess();
      }
    );
  }

  @override
  Widget build(BuildContext context) {
    final ad = ref.watch(_bannerAd);
    
    if (ad != null) {
      return Container(
        width: double.infinity,
        height: ad.size.height.toDouble(),
        alignment: Alignment.center,
        child: AdWidget(ad: ad),
      );
    } else {
      return _blankBox;
    }
  }

}
